from pipelex.cogt.exceptions import CogtError


class AnthropicModelListingError(CogtError):
    pass


class AnthropicSDKUnsupportedError(CogtError):
    pass


class AnthropicWorkerConfigurationError(CogtError):
    pass
